#include <avr/power.h>
#include <avr/sleep.h>
#include <Wire.h>
#include <TimerOne.h>
#include "LED.h"

#define DEBUG
#define BUFFER_SIZE 20
uint8_t I2C_buffer[BUFFER_SIZE];
uint8_t buffer[BUFFER_SIZE];

uint8_t Char_nu=0;//how many byte received

LED5x7 LED;

boolean flag_data_receive;//1 = it successfully receives from the I2C wire.


void setup()
{
	Wire.begin(XADOW_LED_ADDR);                // join i2c bus with address #4
	Wire.onReceive(receiveEvent); // register event
	Serial.begin(38400);           // start serial for output
	LED.init();
	LED.print("start",100);
}

void loop()
{
	if(flag_data_receive)
	{
		copyArray(I2C_buffer,buffer,BUFFER_SIZE);
		uint16_t MovingTime;
		uint8_t str_len;
		String string_disp;
		switch(buffer[0])
		{
			case DISP_CHAR_5X7:
				
				MovingTime = ((uint16_t)buffer[2])<<8;
				MovingTime += buffer[3];
				LED.print(buffer[1],MovingTime);
				break;
			case DISP_STRING:
				str_len = buffer[1];
				for(uint8_t i = 0;i < str_len;i ++)
				{
					string_disp += (char)buffer[i+2];
					Serial.write(string_disp[i]);
				}
				MovingTime = ((uint16_t)buffer[2+str_len])<<8;
				MovingTime += buffer[3+str_len];
				LED.print(string_disp,MovingTime);
				break;
			case SET_DISP_ORIENTATION:
				LED.setOrientation(buffer[1]);
				break;
			case POWER_DOWN:
				powerDown();
				break;
			default:break;
		}
		clearBuffer(buffer,BUFFER_SIZE);
		flag_data_receive = 0;
	}
	LED.printNumber(5);
}

/****************************************/
void receiveEvent(int howMany)
{
	clearBuffer(I2C_buffer,BUFFER_SIZE);
	//input all the received data to I2C_buffer:
	//Char_nu: the number of byte reeived(include the first 2 byte)
	Char_nu=0;
	while(0<Wire.available())
	{
		I2C_buffer[Char_nu]=Wire.read();
		Char_nu++;
	}
#ifdef DEBUG
	Serial.println("the I2C_buffer  is");
	for (unsigned char i =0;i<Char_nu;i++)
	{
		Serial.write(I2C_buffer[i]);
	}
#endif;
	flag_data_receive = 1;
	//parse the data in I2C_buffer;

}
/*************************************************/
void copyArray(uint8_t source[],uint8_t destination[],uint8_t length)
{
	for(uint8_t i = 0; i < length;i ++)
	{
		destination[i] = source[i];
	}
}
/*************************************************/
void clearBuffer(uint8_t buff[],uint8_t size)
{
     for (uint8_t i = 0;i < size;i++)
     {
          buff[i]=0x00;
     }
}
void powerDown()
{
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	attachInterrupt(0, wakUpIsr, LOW);
	cli();
	sleep_enable();
	sei();
	sleep_cpu();
}
void wakUpIsr()
{
	sleep_disable();
	detachInterrupt(0);
}